@@ -34,20 +34,17 @@ import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
 
-import org.antlr.runtime.tree.CommonTree;
 import org.antlr.runtime.tree.Tree;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.commons.lang3.tuple.Pair;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.common.FileUtils;
 import org.apache.hadoop.hive.conf.HiveConf;
-import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
 import org.apache.hadoop.hive.metastore.HiveMetaStore;
 import org.apache.hadoop.hive.metastore.api.Database;
 import org.apache.hadoop.hive.metastore.api.FieldSchema;
 import org.apache.hadoop.hive.metastore.api.Order;
+import org.apache.hadoop.hive.ql.CompilationOpContext;
 import org.apache.hadoop.hive.ql.Context;
 import org.apache.hadoop.hive.ql.ErrorMsg;
 import org.apache.hadoop.hive.ql.QueryProperties;
@@ -75,8 +72,9 @@ import org.apache.hadoop.hive.serde2.io.DateWritable;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspector;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspectorConverters;
 import org.apache.hadoop.hive.serde2.typeinfo.TypeInfo;
-import org.apache.hadoop.hive.serde2.typeinfo.TypeInfoFactory;
 import org.apache.hadoop.hive.serde2.typeinfo.TypeInfoUtils;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import com.google.common.annotations.VisibleForTesting;
 
@@ -85,14 +83,15 @@ import com.google.common.annotations.VisibleForTesting;
  *
  */
 public abstract class BaseSemanticAnalyzer {
-  protected static final Log STATIC_LOG = LogFactory.getLog(BaseSemanticAnalyzer.class.getName());
+  protected static final Logger STATIC_LOG = LoggerFactory.getLogger(BaseSemanticAnalyzer.class.getName());
   protected final Hive db;
   protected final HiveConf conf;
   protected List<Task<? extends Serializable>> rootTasks;
   protected FetchTask fetchTask;
-  protected final Log LOG;
+  protected final Logger LOG;
   protected final LogHelper console;
 
+  protected CompilationOpContext cContext;
   protected Context ctx;
   protected HashMap<String, String> idToTableNameMap;
   protected QueryProperties queryProperties;
@@ -125,6 +124,19 @@ public abstract class BaseSemanticAnalyzer {
    * Columns accessed by updates
    */
   protected ColumnAccessInfo updateColumnAccessInfo;
+  /**
+   * the value of set autocommit true|false
+   * It's an object to make sure it's {@code null} if the parsed statement is
+   * not 'set autocommit...'
+   */
+  private Boolean autoCommitValue;
+
+  public Boolean getAutoCommitValue() {
+    return autoCommitValue;
+  }
+  void setAutoCommitValue(Boolean autoCommit) {
+    autoCommitValue = autoCommit;
+  }
 
 
   public boolean skipAuthorization() {
@@ -190,7 +202,7 @@ public abstract class BaseSemanticAnalyzer {
       this.conf = conf;
       this.db = db;
       rootTasks = new ArrayList<Task<? extends Serializable>>();
-      LOG = LogFactory.getLog(this.getClass().getName());
+      LOG = LoggerFactory.getLogger(this.getClass().getName());
       console = new LogHelper(LOG);
       idToTableNameMap = new HashMap<String, String>();
       inputs = new LinkedHashSet<ReadEntity>();
