@@ -28,12 +28,10 @@ import java.util.concurrent.TimeoutException;
 
 import javax.security.auth.login.LoginException;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
-import org.apache.hadoop.hive.metastore.HiveMetaStoreClient;
-import org.apache.hadoop.hive.metastore.IMetaStoreClient;
 import org.apache.hadoop.hive.metastore.api.MetaException;
 import org.apache.hadoop.hive.ql.exec.FunctionRegistry;
 import org.apache.hadoop.hive.ql.metadata.Hive;
@@ -62,7 +60,7 @@ public class CLIService extends CompositeService implements ICLIService {
     SERVER_VERSION = protocols[protocols.length - 1];
   }
 
-  private final Log LOG = LogFactory.getLog(CLIService.class.getName());
+  private final Logger LOG = LoggerFactory.getLogger(CLIService.class.getName());
 
   private HiveConf hiveConf;
   private SessionManager sessionManager;
@@ -81,7 +79,7 @@ public class CLIService extends CompositeService implements ICLIService {
     this.hiveConf = hiveConf;
     sessionManager = new SessionManager(hiveServer2);
     addService(sessionManager);
-    // If the hadoop cluster is secure and authentication method is kerberos, do a kerberos login for the service from the keytab
+    //  If the hadoop cluster is secure  and authentication method is kerberos, do a kerberos login for the service from the keytab
     if (hiveConf.getVar(ConfVars.HIVE_SERVER2_AUTHENTICATION).equalsIgnoreCase(HiveAuthFactory.AuthTypes.KERBEROS.toString())) {
       try {
         HiveAuthFactory.loginFromKeytab(hiveConf);
@@ -145,19 +143,6 @@ public class CLIService extends CompositeService implements ICLIService {
   @Override
   public synchronized void start() {
     super.start();
-    // Initialize and test a connection to the metastore
-    IMetaStoreClient metastoreClient = null;
-    try {
-      metastoreClient = new HiveMetaStoreClient(hiveConf);
-      metastoreClient.getDatabases("default");
-    } catch (Exception e) {
-      throw new ServiceException("Unable to connect to MetaStore!", e);
-    }
-    finally {
-      if (metastoreClient != null) {
-        metastoreClient.close();
-      }
-    }
   }
 
   @Override
